@@ -3,15 +3,13 @@
 #include "config.h"
 #endif
 #include <schroedinger/schro.h>
-#include <liboil/liboil.h>
 #include <stdlib.h>
 #include <schroedinger/schrocuda.h>
-#ifdef HAVE_ORC
 #include <orc/orc.h>
-#endif
 #include <schroedinger/opengl/schroopengl.h>
 
 extern int _schro_decode_prediction_only;
+extern int _schro_telemetry;
 extern int _schro_dump_enable;
 extern int _schro_motion_ref;
 
@@ -31,10 +29,7 @@ schro_init(void)
 
   inited = TRUE;
 
-  oil_init();
-#ifdef HAVE_ORC
   orc_init();
-#endif
 
   s = getenv ("SCHRO_DEBUG");
   if (s && s[0]) {
@@ -52,6 +47,11 @@ schro_init(void)
     _schro_decode_prediction_only = TRUE;
   }
 
+  s = getenv ("SCHRO_TELEMETRY");
+  if (s && s[0]) {
+    _schro_telemetry = TRUE;
+  }
+
   s = getenv ("SCHRO_MOTION_REF");
   if (s && s[0]) {
     _schro_motion_ref = TRUE;
