@@ -5,11 +5,9 @@
 
 #include <schroedinger/schrobuffer.h>
 #include <schroedinger/schrodebug.h>
-#ifdef HAVE_ORC
 #include <schroedinger/schroorc.h>
-#endif
-#include <liboil/liboil.h>
 #include <string.h>
+#include <orc/orc.h>
 
 static void schro_buffer_free_mem (SchroBuffer * buffer, void *);
 static void schro_buffer_free_subbuffer (SchroBuffer * buffer, void *priv);
@@ -110,11 +108,7 @@ schro_buffer_dup (SchroBuffer * buffer)
   SchroBuffer *dup;
 
   dup = schro_buffer_new_and_alloc (buffer->length);
-#ifdef HAVE_ORC
   orc_memcpy (dup->data, buffer->data, buffer->length);
-#else
-  oil_memcpy (dup->data, buffer->data, buffer->length);
-#endif
 
   return dup;
 }
