@@ -3,8 +3,6 @@
 #include "config.h"
 #endif
 #include <schroedinger/schro.h>
-#include <liboil/liboil.h>
-#include <schroedinger/schrooil.h>
 #include <schroedinger/schrounpack.h>
 #include <string.h>
 
