@@ -3,16 +3,14 @@
 #include "config.h"
 #endif
 #include <schroedinger/schro.h>
-#include <liboil/liboil.h>
 #include <string.h>
-#include <schroedinger/schrooil.h>
-#ifdef HAVE_ORC
 #include <schroedinger/schroorc.h>
-#endif
+#include <orc/orc.h>
 
 extern int _schro_motion_ref;
 
 
+
 SchroMotion *
 schro_motion_new (SchroParams *params, SchroUpsampledFrame *ref1,
     SchroUpsampledFrame *ref2)
@@ -51,9 +49,300 @@ get_ramp (int x, int offset)
   return 1 + (6 * x + offset - 1)/(2*offset - 1);
 }
 
+
+/* runtime orc code generation */
+
+static SchroMotionFuncs motion_funcs[32];
+
+static void
+schro_motion_init_functions (SchroMotion *motion)
+{
+  if (motion_funcs[motion->xblen>>1].block_accumulate == NULL) {
+    OrcProgram *p;
+    OrcCompileResult result;
+
+    p = orc_program_new ();
+    orc_program_set_constant_n (p, motion->xblen);
+    orc_program_set_2d (p);
+    orc_program_set_name (p, "block_acc_Xxn");
+
+    orc_program_add_destination (p, 2, "d1");
+    orc_program_add_source (p, 2, "s1");
+    orc_program_add_source (p, 1, "s2");
+    orc_program_add_temporary (p, 2, "t1");
+
+    orc_program_append (p, "convubw", ORC_VAR_T1, ORC_VAR_S2, ORC_VAR_D1);
+    orc_program_append (p, "mullw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_S1);
+    orc_program_append (p, "addw", ORC_VAR_D1, ORC_VAR_D1, ORC_VAR_T1);
+
+    result = orc_program_compile (p);
+    if (!ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
+      SCHRO_ERROR("compile failed");
+    }
+
+    motion_funcs[motion->xblen/2].block_accumulate = p;
+  }
+
+  if (motion_funcs[motion->xblen>>1].block_accumulate_scaled == NULL) {
+    OrcProgram *p;
+    OrcCompileResult result;
+
+    p = orc_program_new ();
+    orc_program_set_constant_n (p, motion->xblen);
+    orc_program_set_2d (p);
+    orc_program_set_name (p, "block_acc_scaled_Xxn");
+
+    orc_program_add_destination (p, 2, "d1");
+    orc_program_add_source (p, 2, "s1");
+    orc_program_add_source (p, 1, "s2");
+    orc_program_add_parameter (p, 2, "p1");
+    orc_program_add_constant (p, 2, 32, "c1");
+    orc_program_add_constant (p, 2, 6, "c2");
+    orc_program_add_temporary (p, 2, "t1");
+
+    orc_program_append (p, "convubw", ORC_VAR_T1, ORC_VAR_S2, ORC_VAR_D1);
+    orc_program_append (p, "mullw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_P1);
+    orc_program_append (p, "addw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_C1);
+    orc_program_append (p, "shrsw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_C2);
+    orc_program_append (p, "mullw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_S1);
+    orc_program_append (p, "addw", ORC_VAR_D1, ORC_VAR_D1, ORC_VAR_T1);
+
+    result = orc_program_compile (p);
+    if (!ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
+      SCHRO_ERROR("compile failed");
+    }
+
+    motion_funcs[motion->xblen/2].block_accumulate_scaled = p;
+  }
+
+  if (motion_funcs[motion->xblen>>1].block_accumulate_dc == NULL) {
+    OrcProgram *p;
+    OrcCompileResult result;
+
+    p = orc_program_new ();
+    orc_program_set_constant_n (p, motion->xblen);
+    orc_program_set_2d (p);
+    orc_program_set_name (p, "block_acc_dc_Xxn");
+
+    orc_program_add_destination (p, 2, "d1");
+    orc_program_add_source (p, 2, "s1");
+    orc_program_add_parameter (p, 2, "p1");
+    orc_program_add_temporary (p, 2, "t1");
+
+    orc_program_append (p, "mullw", ORC_VAR_T1, ORC_VAR_S1, ORC_VAR_P1);
+    orc_program_append (p, "addw", ORC_VAR_D1, ORC_VAR_D1, ORC_VAR_T1);
+
+    result = orc_program_compile (p);
+    if (!ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
+      SCHRO_ERROR("compile failed");
+    }
+
+    motion_funcs[motion->xblen/2].block_accumulate_dc = p;
+  }
+
+  if (motion_funcs[motion->xblen>>1].block_accumulate_avg == NULL) {
+    OrcProgram *p;
+    OrcCompileResult result;
+
+    p = orc_program_new ();
+    orc_program_set_constant_n (p, motion->xblen);
+    orc_program_set_2d (p);
+    orc_program_set_name (p, "block_acc_avg_Xxn");
+
+    orc_program_add_destination (p, 2, "d1");
+    orc_program_add_source (p, 2, "s1");
+    orc_program_add_source (p, 1, "s2");
+    orc_program_add_source (p, 1, "s3");
+    orc_program_add_temporary (p, 2, "t1");
+    orc_program_add_temporary (p, 1, "t2");
+
+    orc_program_append (p, "avgub", ORC_VAR_T2, ORC_VAR_S2, ORC_VAR_S3);
+    orc_program_append (p, "convubw", ORC_VAR_T1, ORC_VAR_T2, 0);
+    orc_program_append (p, "mullw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_S1);
+    orc_program_append (p, "addw", ORC_VAR_D1, ORC_VAR_D1, ORC_VAR_T1);
+
+    result = orc_program_compile (p);
+    if (!ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
+      SCHRO_ERROR("compile failed");
+    }
+
+    motion_funcs[motion->xblen/2].block_accumulate_avg = p;
+  }
+
+  if (motion_funcs[motion->xblen>>1].block_accumulate_biref == NULL) {
+    OrcProgram *p;
+    OrcCompileResult result;
+
+    p = orc_program_new ();
+    orc_program_set_constant_n (p, motion->xblen);
+    orc_program_set_2d (p);
+    orc_program_set_name (p, "block_acc_biref_Xxn");
+
+    orc_program_add_destination (p, 2, "d1");
+    orc_program_add_source (p, 2, "s1");
+    orc_program_add_source (p, 1, "s2");
+    orc_program_add_source (p, 1, "s3");
+    orc_program_add_parameter (p, 2, "p1");
+    orc_program_add_parameter (p, 2, "p2");
+    orc_program_add_constant (p, 2, 32, "c1");
+    orc_program_add_constant (p, 2, 6, "c2");
+    orc_program_add_temporary (p, 2, "t1");
+    orc_program_add_temporary (p, 2, "t2");
+
+    orc_program_append (p, "convubw", ORC_VAR_T1, ORC_VAR_S2, 0);
+    orc_program_append (p, "mullw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_P1);
+    orc_program_append (p, "convubw", ORC_VAR_T2, ORC_VAR_S3, 0);
+    orc_program_append (p, "mullw", ORC_VAR_T2, ORC_VAR_T2, ORC_VAR_P2);
+    orc_program_append (p, "addw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_T2);
+    orc_program_append (p, "addw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_C1);
+    orc_program_append (p, "shrsw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_C2);
+    orc_program_append (p, "mullw", ORC_VAR_T1, ORC_VAR_T1, ORC_VAR_S1);
+    orc_program_append (p, "addw", ORC_VAR_D1, ORC_VAR_D1, ORC_VAR_T1);
+
+    result = orc_program_compile (p);
+    if (!ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
+      SCHRO_ERROR("compile failed");
+    }
+
+    motion_funcs[motion->xblen/2].block_accumulate_biref = p;
+  }
+}
+
+static void
+orc_multiply_and_acc_Xxn_s16_u8 (int16_t * d1, int d1_stride,
+    const int16_t * s1, int s1_stride, const uint8_t * s2, int s2_stride,
+    int n, int m)
+{
+  OrcExecutor _ex, *ex = &_ex;
+  OrcProgram *p = 0;
+  void (*func) (OrcExecutor *);
+
+  p = motion_funcs[n>>1].block_accumulate;
+  ex->program = p;
+
+  ex->n = n;
+  ORC_EXECUTOR_M(ex) = m;
+  ex->arrays[ORC_VAR_D1] = d1;
+  ex->params[ORC_VAR_D1] = d1_stride;
+  ex->arrays[ORC_VAR_S1] = (void *)s1;
+  ex->params[ORC_VAR_S1] = s1_stride;
+  ex->arrays[ORC_VAR_S2] = (void *)s2;
+  ex->params[ORC_VAR_S2] = s2_stride;
+
+  func = p->code_exec;
+  func (ex);
+}
+
+static void
+orc_multiply_and_acc_scaled_Xxn_s16_u8 (int16_t * d1, int d1_stride,
+    const int16_t * s1, int s1_stride, const uint8_t * s2, int s2_stride,
+    int p1, int n, int m)
+{
+  OrcExecutor _ex, *ex = &_ex;
+  OrcProgram *p = 0;
+  void (*func) (OrcExecutor *);
+
+  p = motion_funcs[n>>1].block_accumulate_scaled;
+  ex->program = p;
+
+  ex->n = n;
+  ORC_EXECUTOR_M(ex) = m;
+  ex->arrays[ORC_VAR_D1] = d1;
+  ex->params[ORC_VAR_D1] = d1_stride;
+  ex->arrays[ORC_VAR_S1] = (void *)s1;
+  ex->params[ORC_VAR_S1] = s1_stride;
+  ex->arrays[ORC_VAR_S2] = (void *)s2;
+  ex->params[ORC_VAR_S2] = s2_stride;
+  ex->params[ORC_VAR_P1] = p1;
+
+  func = p->code_exec;
+  func (ex);
+}
+
+static void
+orc_multiply_and_acc_dc_Xxn_s16_u8 (int16_t * d1, int d1_stride,
+    const int16_t * s1, int s1_stride, int p1, int n, int m)
+{
+  OrcExecutor _ex, *ex = &_ex;
+  OrcProgram *p = 0;
+  void (*func) (OrcExecutor *);
+
+  p = motion_funcs[n>>1].block_accumulate_dc;
+  ex->program = p;
+
+  ex->n = n;
+  ORC_EXECUTOR_M(ex) = m;
+  ex->arrays[ORC_VAR_D1] = d1;
+  ex->params[ORC_VAR_D1] = d1_stride;
+  ex->arrays[ORC_VAR_S1] = (void *)s1;
+  ex->params[ORC_VAR_S1] = s1_stride;
+  ex->params[ORC_VAR_P1] = p1;
+
+  func = p->code_exec;
+  func (ex);
+}
+
+static void
+orc_multiply_and_acc_avg_Xxn_s16_u8 (int16_t * d1, int d1_stride,
+    const int16_t * s1, int s1_stride, const uint8_t * s2, int s2_stride,
+    const uint8_t * s3, int s3_stride, int n, int m)
+{
+  OrcExecutor _ex, *ex = &_ex;
+  OrcProgram *p;
+  void (*func) (OrcExecutor *);
+
+  p = motion_funcs[n>>1].block_accumulate_avg;
+  
+  ex->program = p;
+
+  ex->n = n;
+  ORC_EXECUTOR_M(ex) = m;
+  ex->arrays[ORC_VAR_D1] = d1;
+  ex->params[ORC_VAR_D1] = d1_stride;
+  ex->arrays[ORC_VAR_S1] = (void *)s1;
+  ex->params[ORC_VAR_S1] = s1_stride;
+  ex->arrays[ORC_VAR_S2] = (void *)s2;
+  ex->params[ORC_VAR_S2] = s2_stride;
+  ex->arrays[ORC_VAR_S3] = (void *)s3;
+  ex->params[ORC_VAR_S3] = s3_stride;
+
+  func = p->code_exec;
+  func (ex);
+}
+
+static void
+orc_multiply_and_acc_biref_Xxn_s16_u8 (int16_t * d1, int d1_stride,
+    const int16_t * s1, int s1_stride, const uint8_t * s2, int s2_stride,
+    const uint8_t * s3, int s3_stride, int p1, int p2, int n, int m)
+{
+  OrcExecutor _ex, *ex = &_ex;
+  OrcProgram *p;
+  void (*func) (OrcExecutor *);
+
+  p = motion_funcs[n>>1].block_accumulate_biref;
+  
+  ex->program = p;
+
+  ex->n = n;
+  ORC_EXECUTOR_M(ex) = m;
+  ex->arrays[ORC_VAR_D1] = d1;
+  ex->params[ORC_VAR_D1] = d1_stride;
+  ex->arrays[ORC_VAR_S1] = (void *)s1;
+  ex->params[ORC_VAR_S1] = s1_stride;
+  ex->arrays[ORC_VAR_S2] = (void *)s2;
+  ex->params[ORC_VAR_S2] = s2_stride;
+  ex->arrays[ORC_VAR_S3] = (void *)s3;
+  ex->params[ORC_VAR_S3] = s3_stride;
+  ex->params[ORC_VAR_P1] = p1;
+  ex->params[ORC_VAR_P2] = p2;
+
+  func = p->code_exec;
+  func (ex);
+}
+
 /* motion render (faster) */
 
-void
+static void
 get_block (SchroMotion *motion, int k, int ref, int i, int j, int dx, int dy)
 {
   int px, py;
@@ -106,6 +395,18 @@ get_dc_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   }
 }
 
+static void
+get_ref1_block_simple (SchroMotion *motion, int i, int j, int k, int x, int y)
+{
+  SchroParams *params = motion->params;
+  SchroMotionVector *mv;
+
+  mv = &motion->motion_vectors[j*params->x_num_blocks + i];
+  SCHRO_ASSERT (mv->using_global == FALSE);
+
+  get_block (motion, k, 0, i, j, mv->u.vec.dx[0], mv->u.vec.dy[0]);
+}
+
 static void
 get_ref1_block (SchroMotion *motion, int i, int j, int k, int x, int y)
 {
@@ -123,7 +424,7 @@ get_ref1_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   weight = motion->ref1_weight + motion->ref2_weight;
   shift = motion->ref_weight_precision;
 
-  if (weight == (1<<shift)) {
+  if (motion->oneref_noscale) {
     memcpy (&motion->block, &motion->block_ref[0],
         sizeof(SchroFrameData));
   } else {
@@ -139,6 +440,18 @@ get_ref1_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   }
 }
 
+static void
+get_ref2_block_simple (SchroMotion *motion, int i, int j, int k, int x, int y)
+{
+  SchroParams *params = motion->params;
+  SchroMotionVector *mv;
+
+  mv = &motion->motion_vectors[j*params->x_num_blocks + i];
+  SCHRO_ASSERT (mv->using_global == FALSE);
+
+  get_block (motion, k, 1, i, j, mv->u.vec.dx[1], mv->u.vec.dy[1]);
+}
+
 static void
 get_ref2_block (SchroMotion *motion, int i, int j, int k, int x, int y)
 {
@@ -156,7 +469,7 @@ get_ref2_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   weight = motion->ref1_weight + motion->ref2_weight;
   shift = motion->ref_weight_precision;
 
-  if (weight == (1<<shift)) {
+  if (motion->oneref_noscale) {
     memcpy (&motion->block, &motion->block_ref[1],
         sizeof(SchroFrameData));
   } else {
@@ -172,12 +485,24 @@ get_ref2_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   }
 }
 
+static void
+get_biref_block_simple (SchroMotion *motion, int i, int j, int k, int x, int y)
+{
+  SchroParams *params = motion->params;
+  SchroMotionVector *mv;
+
+  mv = &motion->motion_vectors[j*params->x_num_blocks + i];
+  SCHRO_ASSERT (mv->using_global == FALSE);
+
+  get_block (motion, k, 0, i, j, mv->u.vec.dx[0], mv->u.vec.dy[0]);
+  get_block (motion, k, 1, i, j, mv->u.vec.dx[1], mv->u.vec.dy[1]);
+}
+
 static void
 get_biref_block (SchroMotion *motion, int i, int j, int k, int x, int y)
 {
   SchroParams *params = motion->params;
   SchroMotionVector *mv;
-  int ii, jj;
   int weight0, weight1;
   int shift;
 
@@ -192,84 +517,76 @@ get_biref_block (SchroMotion *motion, int i, int j, int k, int x, int y)
   shift = motion->ref_weight_precision;
 
   memcpy (&motion->block, &motion->alloc_block, sizeof(SchroFrameData));
-  if (weight0 == 1 && weight1 == 1 && shift == 1) {
+  if (motion->simple_weight) {
     switch (motion->xblen) {
       case 8:
-        oil_avg2_8xn_u8(motion->block.data, motion->block.stride,
+        orc_avg2_8xn_u8(motion->block.data, motion->block.stride,
             motion->block_ref[0].data, motion->block_ref[0].stride,
             motion->block_ref[1].data, motion->block_ref[1].stride,
             motion->yblen);
         break;
       case 12:
-        oil_avg2_12xn_u8(motion->block.data, motion->block.stride,
+        orc_avg2_12xn_u8(motion->block.data, motion->block.stride,
             motion->block_ref[0].data, motion->block_ref[0].stride,
             motion->block_ref[1].data, motion->block_ref[1].stride,
             motion->yblen);
         break;
       case 16:
-        oil_avg2_16xn_u8(motion->block.data, motion->block.stride,
+        orc_avg2_16xn_u8(motion->block.data, motion->block.stride,
             motion->block_ref[0].data, motion->block_ref[0].stride,
             motion->block_ref[1].data, motion->block_ref[1].stride,
             motion->yblen);
         break;
       case 32:
-        oil_avg2_32xn_u8(motion->block.data, motion->block.stride,
+        orc_avg2_32xn_u8(motion->block.data, motion->block.stride,
             motion->block_ref[0].data, motion->block_ref[0].stride,
             motion->block_ref[1].data, motion->block_ref[1].stride,
             motion->yblen);
         break;
       default:
-        for(jj=0;jj<motion->yblen;jj++) {
-          uint8_t *d = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
-          uint8_t *s0 = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[0], jj);
-          uint8_t *s1 = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[1], jj);
-          for(ii=0;ii<motion->xblen;ii++) {
-            d[ii] = (s0[ii] + s1[ii] + 1)>>1;
-          }
-        }
+        orc_avg2_nxm_u8(motion->block.data, motion->block.stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
+            motion->xblen, motion->yblen);
         break;
     }
   } else {
-    int16_t as[4];
-    as[0] = weight0;
-    as[1] = weight1;
-    as[2] = (1<<shift)>>1;
-    as[3] = shift;
-
     switch (motion->xblen) {
+#if 0
       case 8:
-        oil_combine2_8xn_u8(motion->block.data, motion->block.stride,
+        orc_combine2_8xn_u8(motion->block.data, motion->block.stride,
             motion->block_ref[0].data, motion->block_ref[0].stride,
             motion->block_ref[1].data, motion->block_ref[1].stride,
-            as, motion->yblen);
+            weight0, weight1, (1<<shift)>>1, shift,
+            motion->yblen);
         break;
       case 12:
-        oil_combine2_16xn_u8(motion->block.data, motion->block.stride,
+        orc_combine2_12xn_u8(motion->block.data, motion->block.stride,
             motion->block_ref[0].data, motion->block_ref[0].stride,
             motion->block_ref[1].data, motion->block_ref[1].stride,
-            as, motion->yblen);
+            weight0, weight1, (1<<shift)>>1, shift,
+            motion->yblen);
         break;
       case 16:
-        oil_combine2_16xn_u8(motion->block.data, motion->block.stride,
+        orc_combine2_16xn_u8(motion->block.data, motion->block.stride,
             motion->block_ref[0].data, motion->block_ref[0].stride,
             motion->block_ref[1].data, motion->block_ref[1].stride,
-            as, motion->yblen);
+            weight0, weight1, (1<<shift)>>1, shift,
+            motion->yblen);
         break;
+#endif
       default:
-        for(jj=0;jj<motion->yblen;jj++) {
-          uint8_t *d = SCHRO_FRAME_DATA_GET_LINE (&motion->block, jj);
-          uint8_t *s0 = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[0], jj);
-          uint8_t *s1 = SCHRO_FRAME_DATA_GET_LINE (&motion->block_ref[1], jj);
-          for(ii=0;ii<motion->xblen;ii++) {
-            d[ii] = ROUND_SHIFT(s0[ii] * weight0 + s1[ii] * weight1, shift);
-          }
-        }
+        orc_combine2_nxm_u8 (motion->block.data, motion->block.stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
+            weight0, weight1, (1<<shift)>>1, shift,
+            motion->xblen, motion->yblen);
         break;
     }
   }
 }
 
-void
+static void
 schro_motion_block_predict_block (SchroMotion *motion, int x, int y, int k,
     int i, int j)
 {
@@ -297,70 +614,104 @@ schro_motion_block_predict_block (SchroMotion *motion, int x, int y, int k,
   }
 }
 
-void
-schro_motion_block_accumulate (SchroMotion *motion, SchroFrameData *comp,
-    int x, int y)
+static void
+schro_motion_block_predict_and_acc (SchroMotion *motion, int x, int y, int k,
+    int i, int j, SchroFrameData *comp)
 {
-  int j;
+  SchroParams *params = motion->params;
+  SchroMotionVector *mv;
 
-  switch (motion->xblen) {
-    case 6:
-      oil_multiply_and_acc_6xn_s16_u8 (
-          SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
-          motion->obmc_weight.data, motion->obmc_weight.stride,
-          motion->block.data, motion->block.stride,
-          motion->yblen);
-      break;
-    case 8:
-      oil_multiply_and_acc_8xn_s16_u8 (
-          SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
-          motion->obmc_weight.data, motion->obmc_weight.stride,
-          motion->block.data, motion->block.stride,
-          motion->yblen);
-      break;
-    case 12:
-      oil_multiply_and_acc_12xn_s16_u8 (
-          SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
-          motion->obmc_weight.data, motion->obmc_weight.stride,
-          motion->block.data, motion->block.stride,
-          motion->yblen);
-      break;
-    case 16:
-      oil_multiply_and_acc_16xn_s16_u8 (
-          SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
-          motion->obmc_weight.data, motion->obmc_weight.stride,
-          motion->block.data, motion->block.stride,
-          motion->yblen);
-      break;
-    case 24:
-      oil_multiply_and_acc_24xn_s16_u8 (
-          SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
-          motion->obmc_weight.data, motion->obmc_weight.stride,
-          motion->block.data, motion->block.stride,
-          motion->yblen);
-      break;
-    default:
-      for(j=0;j<motion->yblen;j++) {
-#ifdef HAVE_ORC
-        orc_multiply_and_add_s16_u8 (
-            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y + j),
-            SCHRO_FRAME_DATA_GET_LINE (&motion->obmc_weight, j),
-            SCHRO_FRAME_DATA_GET_LINE (&motion->block, j),
-            motion->xblen);
-#else
-        oil_multiply_and_add_s16_u8 (
-            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y + j),
-            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y + j),
-            SCHRO_FRAME_DATA_GET_LINE (&motion->obmc_weight, j),
-            SCHRO_FRAME_DATA_GET_LINE (&motion->block, j),
-            motion->xblen);
-#endif
-      }
-      break;
+  mv = &motion->motion_vectors[j*params->x_num_blocks + i];
+
+  if (motion->simple_weight) {
+    switch (mv->pred_mode) {
+      case 0:
+        orc_multiply_and_acc_dc_Xxn_s16_u8 (
+            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
+            motion->obmc_weight.data, motion->obmc_weight.stride,
+            mv->u.dc.dc[k]+128,
+            motion->xblen, motion->yblen);
+        break;
+      case 1:
+        get_ref1_block_simple (motion, i, j, k, x, y);
+        orc_multiply_and_acc_Xxn_s16_u8 (
+            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
+            motion->obmc_weight.data, motion->obmc_weight.stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->xblen, motion->yblen);
+        break;
+      case 2:
+        get_ref2_block_simple (motion, i, j, k, x, y);
+        orc_multiply_and_acc_Xxn_s16_u8 (
+            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
+            motion->obmc_weight.data, motion->obmc_weight.stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
+            motion->xblen, motion->yblen);
+        break;
+      case 3:
+        get_biref_block_simple (motion, i, j, k, x, y);
+        orc_multiply_and_acc_avg_Xxn_s16_u8 (
+            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
+            motion->obmc_weight.data, motion->obmc_weight.stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
+            motion->xblen, motion->yblen);
+        break;
+      default:
+        SCHRO_ASSERT(0);
+        break;
+    }
+  } else {
+    int weight0, weight1, shift;
+
+    weight0 = motion->ref1_weight;
+    weight1 = motion->ref2_weight;
+    shift = motion->ref_weight_precision;
+
+    switch (mv->pred_mode) {
+      case 0:
+        orc_multiply_and_acc_dc_Xxn_s16_u8 (
+            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
+            motion->obmc_weight.data, motion->obmc_weight.stride,
+            mv->u.dc.dc[k]+128,
+            motion->xblen, motion->yblen);
+        break;
+      case 1:
+        get_ref1_block_simple (motion, i, j, k, x, y);
+        orc_multiply_and_acc_scaled_Xxn_s16_u8 (
+            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
+            motion->obmc_weight.data, motion->obmc_weight.stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            (weight0 + weight1)<<(6-shift),
+            motion->xblen, motion->yblen);
+        break;
+      case 2:
+        get_ref2_block_simple (motion, i, j, k, x, y);
+        orc_multiply_and_acc_scaled_Xxn_s16_u8 (
+            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
+            motion->obmc_weight.data, motion->obmc_weight.stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
+            (weight0 + weight1)<<(6-shift),
+            motion->xblen, motion->yblen);
+        break;
+      case 3:
+        get_biref_block_simple (motion, i, j, k, x, y);
+        orc_multiply_and_acc_biref_Xxn_s16_u8 (
+            SCHRO_FRAME_DATA_GET_PIXEL_S16 (comp, x, y), comp->stride,
+            motion->obmc_weight.data, motion->obmc_weight.stride,
+            motion->block_ref[0].data, motion->block_ref[0].stride,
+            motion->block_ref[1].data, motion->block_ref[1].stride,
+            weight0<<(6-shift), weight1<<(6-shift),
+            motion->xblen, motion->yblen);
+        break;
+      default:
+        SCHRO_ASSERT(0);
+        break;
+    }
   }
 }
 
-void
+static void
 schro_motion_block_accumulate_slow (SchroMotion *motion, SchroFrameData *comp,
     int x, int y)
 {
@@ -444,7 +795,8 @@ schro_motion_init_obmc_weight (SchroMotion *motion)
 }
 
 void
-schro_motion_render (SchroMotion *motion, SchroFrame *dest)
+schro_motion_render (SchroMotion *motion, SchroFrame *dest,
+    SchroFrame *addframe, int add, SchroFrame *output_frame)
 {
   int i, j;
   int x, y;
@@ -452,26 +804,23 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
   SchroParams *params = motion->params;
   int max_x_blocks;
   int max_y_blocks;
-#ifndef HAVE_ORC
-  int16_t zero = 0;
-#endif
 
 #ifdef ENABLE_MOTION_REF
   if (_schro_motion_ref) {
-    schro_motion_render_ref (motion, dest);
+    schro_motion_render_ref (motion, dest, addframe, add, output_frame);
     return;
   }
 #endif
 
-  if (schro_motion_render_fast_allowed (motion)) {
-    schro_motion_render_fast (motion, dest);
+  if (0 && schro_motion_render_fast_allowed (motion)) {
+    schro_motion_render_fast (motion, dest, addframe, add, output_frame);
     return;
   }
 
   if (params->have_global_motion) {
 #ifdef ENABLE_MOTION_REF
     SCHRO_WARNING ("global motion enabled, using reference motion renderer");
-    schro_motion_render_ref (motion, dest);
+    schro_motion_render_ref (motion, dest, addframe, add, output_frame);
     return;
 #else
     SCHRO_ERROR ("global motion enabled, probably will crash");
@@ -496,7 +845,7 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
 #ifdef ENABLE_MOTION_REF
       SCHRO_WARNING ("block size (%dx%d) larger than minimum frame extension %d, using reference motion renderer",
           params->xblen_luma, params->yblen_luma, min_extension);
-      schro_motion_render_ref (motion, dest);
+      schro_motion_render_ref (motion, dest, addframe, add, output_frame);
       return;
 #else
       SCHRO_ERROR ("block size (%dx%d) larger than minimum frame extension %d, probably will crash",
@@ -517,6 +866,12 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
 
   for (k=0;k<3;k++){
     SchroFrameData *comp = dest->components + k;
+    SchroFrameData *acomp = addframe->components + k;
+    SchroFrameData *ocomp = NULL;
+    
+    if (output_frame) {
+      ocomp = output_frame->components + k;
+    }
 
     if (k == 0) {
       motion->xbsep = params->xbsep_luma;
@@ -559,17 +914,18 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
     motion->alloc_block_ref[1].width = motion->xblen;
     motion->alloc_block_ref[1].height = motion->yblen;
 
-    schro_motion_init_obmc_weight (motion);
-
-    for(j=0;j<comp->height;j++){
-#ifdef HAVE_ORC
-      orc_splat_s16_ns (SCHRO_FRAME_DATA_GET_LINE(comp, j), 0,
-          comp->width);
-#else
-      oil_splat_s16_ns (SCHRO_FRAME_DATA_GET_LINE(comp, j), &zero,
-          comp->width);
-#endif
+    if (motion->ref1_weight == 1 && motion->ref2_weight == 1 &&
+        motion->ref_weight_precision == 1) {
+      motion->simple_weight = TRUE;
     }
+    if (motion->ref1_weight + motion->ref2_weight ==
+        (1<<motion->ref_weight_precision)) {
+      motion->oneref_noscale = TRUE;
+    }
+
+    schro_motion_init_obmc_weight (motion);
+    schro_motion_init_functions (motion);
+    //schro_motion_set_block_accumulate (motion);
 
     max_x_blocks = MIN(params->x_num_blocks - 1,
         (motion->width - motion->xoffset)/motion->xbsep);
@@ -577,6 +933,8 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
         (motion->height - motion->yoffset)/motion->ybsep);
 
     j = 0;
+    orc_splat_s16_2d (SCHRO_FRAME_DATA_GET_LINE(comp, 0), comp->stride,
+        0, comp->width, motion->ybsep + motion->yoffset);
     for(i=0;i<params->x_num_blocks;i++){
       x = motion->xbsep * i - motion->xoffset;
       y = motion->ybsep * j - motion->yoffset;
@@ -584,8 +942,22 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
       schro_motion_block_predict_block (motion, x, y, k, i, j);
       schro_motion_block_accumulate_slow (motion, comp, x, y);
     }
+    if (add) {
+      orc_rrshift6_add_s16_2d (
+          SCHRO_FRAME_DATA_GET_LINE(ocomp, 0), ocomp->stride,
+          SCHRO_FRAME_DATA_GET_LINE(acomp, 0), acomp->stride,
+          SCHRO_FRAME_DATA_GET_LINE(comp, 0), comp->stride,
+          motion->width, motion->ybsep - motion->yoffset);
+    } else {
+      orc_rrshift6_sub_s16_2d (
+          SCHRO_FRAME_DATA_GET_LINE(acomp, 0), acomp->stride,
+          SCHRO_FRAME_DATA_GET_LINE(comp, 0), comp->stride,
+          motion->width, motion->ybsep - motion->yoffset);
+    }
     for(j=1;j<max_y_blocks;j++){
       y = motion->ybsep * j - motion->yoffset;
+      orc_splat_s16_2d (SCHRO_FRAME_DATA_GET_LINE(comp, y + motion->yoffset*2),
+          comp->stride, 0, comp->width, motion->ybsep);
 
       i = 0;
       {
@@ -598,8 +970,7 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
       for(i=1;i<max_x_blocks;i++){
         x = motion->xbsep * i - motion->xoffset;
 
-        schro_motion_block_predict_block (motion, x, y, k, i, j);
-        schro_motion_block_accumulate (motion, comp, x, y);
+        schro_motion_block_predict_and_acc (motion, x, y, k, i, j, comp);
       }
 
       for(;i<params->x_num_blocks;i++){
@@ -608,30 +979,60 @@ schro_motion_render (SchroMotion *motion, SchroFrame *dest)
         schro_motion_block_predict_block (motion, x, y, k, i, j);
         schro_motion_block_accumulate_slow (motion, comp, x, y);
       }
+      if (add) {
+        orc_rrshift6_add_s16_2d (
+            SCHRO_FRAME_DATA_GET_LINE(ocomp, y), ocomp->stride,
+            SCHRO_FRAME_DATA_GET_LINE(acomp, y), acomp->stride,
+            SCHRO_FRAME_DATA_GET_LINE(comp, y),
+            comp->stride, motion->width, motion->ybsep);
+      } else {
+        orc_rrshift6_sub_s16_2d (
+            SCHRO_FRAME_DATA_GET_LINE(acomp, y), acomp->stride,
+            SCHRO_FRAME_DATA_GET_LINE(comp, y),
+            comp->stride, motion->width, motion->ybsep);
+      }
     }
     for(j=max_y_blocks;j<params->y_num_blocks;j++){
       y = motion->ybsep * j - motion->yoffset;
+      orc_splat_s16_2d (SCHRO_FRAME_DATA_GET_LINE(comp, y + motion->yoffset*2),
+          comp->stride, 0, comp->width,
+          CLAMP(comp->height - (y+motion->yoffset*2), 0, motion->ybsep));
       for(i=0;i<params->x_num_blocks;i++){
         x = motion->xbsep * i - motion->xoffset;
 
         schro_motion_block_predict_block (motion, x, y, k, i, j);
         schro_motion_block_accumulate_slow (motion, comp, x, y);
       }
+      if (add) {
+        orc_rrshift6_add_s16_2d (
+            SCHRO_FRAME_DATA_GET_LINE(ocomp, y), ocomp->stride,
+            SCHRO_FRAME_DATA_GET_LINE(acomp, y), acomp->stride,
+            SCHRO_FRAME_DATA_GET_LINE(comp, y),
+            comp->stride, comp->width,
+            CLAMP(comp->height - y, 0, motion->ybsep));
+      } else {
+        orc_rrshift6_sub_s16_2d (
+            SCHRO_FRAME_DATA_GET_LINE(acomp, y), acomp->stride,
+            SCHRO_FRAME_DATA_GET_LINE(comp, y),
+            comp->stride, comp->width,
+            CLAMP(comp->height - y, 0, motion->ybsep));
+      }
     }
 
-
-    for(j=0;j<comp->height;j++){
-#ifdef HAVE_ORC
-      orc_rrshift6_s16_ip (SCHRO_FRAME_DATA_GET_LINE(comp, j), motion->width);
-#else
-      int16_t as[2] = { 1, 1 };
-
-      as[1] = 6;
-      as[0] = ((1<<as[1])>>1) - (128<<6);
-
-      oil_add_const_rshift_s16 (SCHRO_FRAME_DATA_GET_LINE(comp, j),
-          SCHRO_FRAME_DATA_GET_LINE(comp, j), as, motion->width);
-#endif
+    y = params->y_num_blocks * motion->ybsep - motion->yoffset;
+    if (add) {
+      orc_rrshift6_add_s16_2d (
+          SCHRO_FRAME_DATA_GET_LINE(ocomp, y), ocomp->stride,
+          SCHRO_FRAME_DATA_GET_LINE(acomp, y), acomp->stride,
+          SCHRO_FRAME_DATA_GET_LINE(comp, y),
+          comp->stride, comp->width,
+          CLAMP(comp->height - y, 0, motion->ybsep));
+    } else {
+      orc_rrshift6_sub_s16_2d (
+          SCHRO_FRAME_DATA_GET_LINE(acomp, y), acomp->stride,
+          SCHRO_FRAME_DATA_GET_LINE(comp, y),
+          comp->stride, comp->width,
+          CLAMP(comp->height - y, 0, motion->ybsep));
     }
 
     schro_free (motion->alloc_block.data);
@@ -741,6 +1142,61 @@ median3(int a, int b, int c)
   }
 }
 
+void
+schro_mf_vector_prediction (SchroMotionField* mf,
+    int x, int y, int* pred_x, int* pred_y, int mode)
+{
+  int x_num_blocks;
+  SchroMotionVector* mv;
+  int vx[3], vy[3];
+  int n = 0;
+  int ref = mode - 1;
+
+  SCHRO_ASSERT(mf && pred_x && pred_y);
+  SCHRO_ASSERT (1 == mode || 2 == mode);
+
+  x_num_blocks = mf->x_num_blocks;
+
+  if (0<x) {
+    mv = &mf->motion_vectors[y * x_num_blocks + x - 1];
+    vx[n] = mv->u.vec.dx[ref];
+    vy[n] = mv->u.vec.dy[ref];
+    ++n;
+  }
+  if (0<y) {
+    mv = &mf->motion_vectors[(y-1) * x_num_blocks + x];
+    vx[n] = mv->u.vec.dx[ref];
+    vy[n] = mv->u.vec.dy[ref];
+    ++n;
+  }
+  if (0<x && 0<y) {
+    mv = &mf->motion_vectors[(y-1) * x_num_blocks + x - 1];
+    vx[n] = mv->u.vec.dx[ref];
+    vy[n] = mv->u.vec.dy[ref];
+    ++n;
+  }
+  switch(n) {
+    case 0:
+      *pred_x = 0;
+      *pred_y = 0;
+      break;
+    case 1:
+      *pred_x = vx[0];
+      *pred_y = vy[0];
+      break;
+    case 2:
+      *pred_x = (vx[0] + vx[1] + 1)>>1;
+      *pred_y = (vy[0] + vy[1] + 1)>>1;
+      break;
+    case 3:
+      *pred_x = median3(vx[0], vx[1], vx[2]);
+      *pred_y = median3(vy[0], vy[1], vy[2]);
+      break;
+    default:
+      SCHRO_ASSERT(0);
+  }
+}
+
 void
 schro_motion_vector_prediction (SchroMotion *motion,
     int x, int y, int *pred_x, int *pred_y, int mode)
@@ -933,6 +1389,9 @@ schro_motion_verify (SchroMotion *motion)
             return 0;
           }
           break;
+        default:
+          SCHRO_ASSERT(0);
+          break;
       }
 
       if (params->have_global_motion == FALSE) {
