@@ -4,11 +4,11 @@
 #endif
 
 #include <string.h>
-#include <liboil/liboil.h>
 
 #include <schroedinger/schropack.h>
 #include <schroedinger/schro.h>
-#include <schroedinger/schrooil.h>
+#include <schroedinger/schroorc.h>
+#include <orc/orc.h>
 
 
 SchroPack *
@@ -90,7 +90,7 @@ schro_pack_sync (SchroPack *pack)
 }
 
 void
-schro_pack_append (SchroPack *pack, uint8_t *data, int len)
+schro_pack_append (SchroPack *pack, const uint8_t *data, int len)
 {
   if (pack->shift != 7) {
     SCHRO_ERROR ("appending to unsyncronized pack");
@@ -98,11 +98,7 @@ schro_pack_append (SchroPack *pack, uint8_t *data, int len)
 
   SCHRO_ASSERT(pack->n + len <= pack->buffer->length);
 
-#ifdef HAVE_ORC
-  orc_memcpy (pack->buffer->data + pack->n, data, len);
-#else
-  oil_memcpy (pack->buffer->data + pack->n, data, len);
-#endif
+  orc_memcpy (pack->buffer->data + pack->n, (void *)data, len);
   pack->n += len;
 }
 
