@@ -8,10 +8,11 @@
 #include "schrovirtframe.h"
 #include <schroedinger/schro.h>
 #include <schroedinger/schroutils.h>
-#include <liboil/liboil.h>
-#include <schroedinger/schrooil.h>
 #include <string.h>
 #include <math.h>
+#include <orc/orc.h>
+
+#include <schroedinger/schroorc.h>
 
 
 SchroFrame *
@@ -51,7 +52,7 @@ schro_frame_new_virtual (SchroMemoryDomain *domain, SchroFrameFormat format,
 
     frame->regions[0] = malloc (frame->components[0].stride * SCHRO_FRAME_CACHE_SIZE);
     for(i=0;i<SCHRO_FRAME_CACHE_SIZE;i++){
-      frame->cached_lines[0][i] = -1;
+      frame->cached_lines[0][i] = 0;
     }
     frame->is_virtual = TRUE;
 
@@ -112,7 +113,7 @@ schro_frame_new_virtual (SchroMemoryDomain *domain, SchroFrameFormat format,
 
     frame->regions[i] = malloc (comp->stride * SCHRO_FRAME_CACHE_SIZE);
     for(j=0;j<SCHRO_FRAME_CACHE_SIZE;j++){
-      frame->cached_lines[i][j] = -1;
+      frame->cached_lines[i][j] = 0;
     }
   }
   frame->is_virtual = TRUE;
@@ -120,13 +121,35 @@ schro_frame_new_virtual (SchroMemoryDomain *domain, SchroFrameFormat format,
   return frame;
 }
 
+static void
+schro_virt_frame_prep_cache_line (SchroFrame *frame, int component, int i)
+{
+  int j;
+
+  if (i < frame->cache_offset[component]) {
+    SCHRO_ERROR ("cache failure: %d outside [%d,%d]", i,
+        frame->cache_offset[component],
+        frame->cache_offset[component] + SCHRO_FRAME_CACHE_SIZE - 1);
+
+    frame->cache_offset[component] = i;
+    for(j=0;j<SCHRO_FRAME_CACHE_SIZE; j++) {
+      frame->cached_lines[component][j] = 0;
+    }
+  }
+
+  while (i >= frame->cache_offset[component] + SCHRO_FRAME_CACHE_SIZE) {
+    j = frame->cache_offset[component] & (SCHRO_FRAME_CACHE_SIZE - 1);
+    frame->cached_lines[component][j] = 0;
+
+    frame->cache_offset[component]++;
+  }
+}
+
 void *
-schro_virt_frame_get_line (SchroFrame *frame, int component, int i)
+schro_virt_frame_get_line_unrendered (SchroFrame *frame, int component, int i)
 {
   SchroFrameData *comp = &frame->components[component];
   int j;
-  int min;
-  int min_j;
 
   //SCHRO_ASSERT(i >= 0);
   //SCHRO_ASSERT(i < comp->height);
@@ -135,27 +158,48 @@ schro_virt_frame_get_line (SchroFrame *frame, int component, int i)
     return SCHRO_FRAME_DATA_GET_LINE(&frame->components[component], i);
   }
 
-  for(j=0;j<SCHRO_FRAME_CACHE_SIZE;j++){
-    if (frame->cached_lines[component][j] == i) {
-      return SCHRO_OFFSET(frame->regions[component], comp->stride * j);
-    }
+  schro_virt_frame_prep_cache_line (frame, component, i);
+  j = i & (SCHRO_FRAME_CACHE_SIZE - 1);
+
+  return SCHRO_OFFSET (frame->regions[component], comp->stride * j);
+}
+
+void *
+schro_virt_frame_get_line (SchroFrame *frame, int component, int i)
+{
+  SchroFrameData *comp = &frame->components[component];
+  int j;
+
+  //SCHRO_ASSERT(i >= 0);
+  //SCHRO_ASSERT(i < comp->height);
+
+  if (!frame->is_virtual) {
+    return SCHRO_FRAME_DATA_GET_LINE(&frame->components[component], i);
   }
 
-  min_j = 0;
-  min = frame->cached_lines[component][0];
-  for(j=1;j<SCHRO_FRAME_CACHE_SIZE;j++){
-    if (frame->cached_lines[component][j] < min) {
-      min = frame->cached_lines[component][j];
-      min_j = j;
-    }
+  schro_virt_frame_prep_cache_line (frame, component, i);
+  j = i & (SCHRO_FRAME_CACHE_SIZE - 1);
+
+  if (!frame->cached_lines[component][j]) {
+    schro_virt_frame_render_line (frame,
+        SCHRO_OFFSET (frame->regions[component], comp->stride * j), component, i);
+    frame->cached_lines[component][j] = 1;
   }
-  frame->cached_lines[component][min_j] = i;
 
-  schro_virt_frame_render_line (frame,
-      SCHRO_OFFSET(frame->regions[component], comp->stride * min_j),
-      component, i);
+  return SCHRO_OFFSET (frame->regions[component], comp->stride * j);
+}
 
-  return SCHRO_OFFSET(frame->regions[component], comp->stride * min_j);
+void
+schro_virt_frame_set_line_rendered (SchroFrame *frame, int component, int i)
+{
+  int j;
+
+  //SCHRO_ASSERT(i >= 0);
+  //SCHRO_ASSERT(i < comp->height);
+  //SCHRO_ASSERT(frame->is_virtual);
+
+  j = i & (SCHRO_FRAME_CACHE_SIZE - 1);
+  frame->cached_lines[component][j] = 1;
 }
 
 void
@@ -174,18 +218,10 @@ copy (SchroFrame *frame, void *_dest, int component, int i)
   src = schro_virt_frame_get_line (frame, component, i);
   switch (SCHRO_FRAME_FORMAT_DEPTH(frame->format)) {
     case SCHRO_FRAME_FORMAT_DEPTH_U8:
-#ifdef HAVE_ORC
       orc_memcpy (dest, src, frame->components[component].width);
-#else
-      oil_memcpy (dest, src, frame->components[component].width);
-#endif
       break;
     case SCHRO_FRAME_FORMAT_DEPTH_S16:
-#ifdef HAVE_ORC
       orc_memcpy (dest, src, frame->components[component].width * 2);
-#else
-      oil_memcpy (dest, src, frame->components[component].width * 2);
-#endif
       break;
     default:
       SCHRO_ASSERT(0);
@@ -576,39 +612,21 @@ unpack_yuyv (SchroFrame *frame, void *_dest, int component, int i)
 {
   uint8_t *dest = _dest;
   uint8_t *src;
-#ifndef HAVE_ORC
-  int j;
-#endif
 
   src = schro_virt_frame_get_line (frame->virt_frame1, 0, i);
 
   switch (component) {
     case 0:
-#ifdef HAVE_ORC
       orc_unpack_yuyv_y (dest, (void *)src, frame->width);
-#else
-      for(j=0;j<frame->width;j++){
-        dest[j] = src[j*2];
-      }
-#endif
       break;
     case 1:
-#ifdef HAVE_ORC
       orc_unpack_yuyv_u (dest, (void *)src, frame->width/2);
-#else
-      for(j=0;j<frame->width/2;j++){
-        dest[j] = src[j*4 + 1];
-      }
-#endif
       break;
     case 2:
-#ifdef HAVE_ORC
       orc_unpack_yuyv_v (dest, (void *)src, frame->width/2);
-#else
-      for(j=0;j<frame->width/2;j++){
-        dest[j] = src[j*4 + 3];
-      }
-#endif
+      break;
+    default:
+      SCHRO_ASSERT(0);
   }
 }
 
@@ -617,39 +635,21 @@ unpack_uyvy (SchroFrame *frame, void *_dest, int component, int i)
 {
   uint8_t *dest = _dest;
   uint8_t *src;
-#ifndef HAVE_ORC
-  int j;
-#endif
 
   src = schro_virt_frame_get_line (frame->virt_frame1, 0, i);
 
   switch (component) {
     case 0:
-#ifdef HAVE_ORC
       orc_unpack_uyvy_y (dest, (void *)src, frame->width);
-#else
-      for(j=0;j<frame->width;j++){
-        dest[j] = src[j*2 + 1];
-      }
-#endif
       break;
     case 1:
-#ifdef HAVE_ORC
       orc_unpack_uyvy_u (dest, (void *)src, frame->width/2);
-#else
-      for(j=0;j<frame->width/2;j++){
-        dest[j] = src[j*4 + 0];
-      }
-#endif
       break;
     case 2:
-#ifdef HAVE_ORC
       orc_unpack_uyvy_v (dest, (void *)src, frame->width/2);
-#else
-      for(j=0;j<frame->width/2;j++){
-        dest[j] = src[j*4 + 2];
-      }
-#endif
+      break;
+    default:
+      SCHRO_ASSERT(0);
   }
 }
 
@@ -677,6 +677,9 @@ unpack_ayuv (SchroFrame *frame, void *_dest, int component, int i)
       for(j=0;j<frame->width;j++){
         dest[j] = src[j*4 + 3];
       }
+      break;
+    default:
+      SCHRO_ASSERT(0);
   }
 }
 
@@ -752,6 +755,8 @@ unpack_v210 (SchroFrame *frame, void *_dest, int component, int i)
         dest[j*3 + 2] = ((READ_UINT32_LE (src + j*16 + 12) >> 10)&0x3ff) >> 2;
       }
       break;
+    default:
+      SCHRO_ASSERT(0);
   }
 }
 
@@ -779,6 +784,9 @@ unpack_v216 (SchroFrame *frame, void *_dest, int component, int i)
       for(j=0;j<frame->width/2;j++){
         dest[j] = src[j*8 + 4 + 1];
       }
+      break;
+    default:
+      SCHRO_ASSERT(0);
   }
 }
 
@@ -834,11 +842,7 @@ pack_yuyv (SchroFrame *frame, void *_dest, int component, int i)
   src_u = schro_virt_frame_get_line (frame->virt_frame1, 1, i);
   src_v = schro_virt_frame_get_line (frame->virt_frame1, 2, i);
 
-#ifdef HAVE_ORC
   orc_packyuyv (dest, src_y, src_u, src_v, frame->width/2);
-#else
-  oil_packyuyv (dest, src_y, src_u, src_v, frame->width/2);
-#endif
 }
 
 
@@ -1259,11 +1263,7 @@ convert_444_422 (SchroFrame *frame, void *_dest, int component, int i)
   src = schro_virt_frame_get_line (frame->virt_frame1, component, i);
 
   if (component == 0) {
-#ifdef HAVE_ORC
     orc_memcpy (dest, src, frame->width);
-#else
-    oil_memcpy (dest, src, frame->width);
-#endif
   } else {
     for(j=0;j<frame->components[component].width;j++){
       dest[j] = src[j*2];
@@ -1280,11 +1280,7 @@ convert_444_420 (SchroFrame *frame, void *_dest, int component, int i)
 
   if (component == 0) {
     src = schro_virt_frame_get_line (frame->virt_frame1, component, i);
-#ifdef HAVE_ORC
     orc_memcpy (dest, src, frame->components[component].width);
-#else
-    oil_memcpy (dest, src, frame->components[component].width);
-#endif
   } else {
     src = schro_virt_frame_get_line (frame->virt_frame1, component, i*2);
     for(j=0;j<frame->components[component].width;j++){
@@ -1304,11 +1300,7 @@ convert_422_420 (SchroFrame *frame, void *_dest, int component, int i)
   } else {
     src = schro_virt_frame_get_line (frame->virt_frame1, component, i*2);
   }
-#ifdef HAVE_ORC
   orc_memcpy (dest, src, frame->components[component].width);
-#else
-  oil_memcpy (dest, src, frame->components[component].width);
-#endif
 }
 
 /* up */
@@ -1323,11 +1315,7 @@ convert_422_444 (SchroFrame *frame, void *_dest, int component, int i)
   src = schro_virt_frame_get_line (frame->virt_frame1, component, i);
 
   if (component == 0) {
-#ifdef HAVE_ORC
     orc_memcpy (dest, src, frame->width);
-#else
-    oil_memcpy (dest, src, frame->width);
-#endif
   } else {
     for(j=0;j<frame->components[component].width;j++){
       dest[j] = src[j>>1];
@@ -1344,11 +1332,7 @@ convert_420_444 (SchroFrame *frame, void *_dest, int component, int i)
 
   if (component == 0) {
     src = schro_virt_frame_get_line (frame->virt_frame1, component, i);
-#ifdef HAVE_ORC
     orc_memcpy (dest, src, frame->components[component].width);
-#else
-    oil_memcpy (dest, src, frame->components[component].width);
-#endif
   } else {
     src = schro_virt_frame_get_line (frame->virt_frame1, component, i>>1);
     for(j=0;j<frame->components[component].width;j++){
@@ -1368,11 +1352,7 @@ convert_420_422 (SchroFrame *frame, void *_dest, int component, int i)
   } else {
     src = schro_virt_frame_get_line (frame->virt_frame1, component, i>>1);
   }
-#ifdef HAVE_ORC
   orc_memcpy (dest, src, frame->components[component].width);
-#else
-  oil_memcpy (dest, src, frame->components[component].width);
-#endif
 }
 
 SchroFrame *
@@ -1529,19 +1509,9 @@ convert_u8_s16 (SchroFrame *frame, void *_dest, int component, int i)
 {
   uint8_t *dest = _dest;
   int16_t *src;
-#ifndef HAVE_ORC
-  int16_t c = 128;
-#endif
 
   src = schro_virt_frame_get_line (frame->virt_frame1, component, i);
-#ifdef HAVE_ORC
   orc_offsetconvert_u8_s16 (dest, src, frame->components[component].width);
-#else
-  oil_addc_s16 (frame->virt_priv, src, &c,
-      frame->components[component].width);
-  oil_convert_u8_s16 (dest, frame->virt_priv,
-      frame->components[component].width);
-#endif
 }
 
 SchroFrame *
@@ -1565,18 +1535,10 @@ convert_s16_u8 (SchroFrame *frame, void *_dest, int component, int i)
 {
   int16_t *dest = _dest;
   uint8_t *src;
-#ifndef HAVE_ORC
-  int16_t c = -128;
-#endif
 
   src = schro_virt_frame_get_line (frame->virt_frame1, component, i);
 
-#ifdef HAVE_ORC
   orc_offsetconvert_s16_u8 (dest, src, frame->components[component].width);
-#else
-  oil_convert_s16_u8 (dest, src, frame->components[component].width);
-  oil_addc_s16 (dest, dest, &c, frame->components[component].width);
-#endif
 }
 
 SchroFrame *
@@ -1601,11 +1563,7 @@ crop_u8 (SchroFrame *frame, void *_dest, int component, int i)
   uint8_t *src;
 
   src = schro_virt_frame_get_line (frame->virt_frame1, component, i);
-#ifdef HAVE_ORC
   orc_memcpy (dest, src, frame->components[component].width);
-#else
-  oil_memcpy (dest, src, frame->components[component].width);
-#endif
 }
 
 static void
@@ -1615,11 +1573,7 @@ crop_s16 (SchroFrame *frame, void *_dest, int component, int i)
   int16_t *src;
 
   src = schro_virt_frame_get_line (frame->virt_frame1, component, i);
-#ifdef HAVE_ORC
   orc_memcpy (dest, src, frame->components[component].width * sizeof(int16_t));
-#else
-  oil_memcpy (dest, src, frame->components[component].width * sizeof(int16_t));
-#endif
 }
 
 SchroFrame *
@@ -1658,19 +1612,11 @@ edge_extend_u8 (SchroFrame *frame, void *_dest, int component, int i)
 
   src = schro_virt_frame_get_line (frame->virt_frame1, component,
       MIN(i,srcframe->components[component].height-1));
-#ifdef HAVE_ORC
   orc_memcpy (dest, src, srcframe->components[component].width);
   orc_splat_u8_ns (dest + srcframe->components[component].width,
       dest[srcframe->components[component].width - 1],
       frame->components[component].width -
       srcframe->components[component].width);
-#else
-  oil_memcpy (dest, src, srcframe->components[component].width);
-  oil_splat_u8_ns (dest + srcframe->components[component].width,
-      dest + srcframe->components[component].width - 1,
-      frame->components[component].width -
-      srcframe->components[component].width);
-#endif
 }
 
 static void
@@ -1682,21 +1628,12 @@ edge_extend_s16 (SchroFrame *frame, void *_dest, int component, int i)
 
   src = schro_virt_frame_get_line (frame->virt_frame1, component,
       MIN(i,srcframe->components[component].height-1));
-#ifdef HAVE_ORC
   orc_memcpy (dest, src,
       srcframe->components[component].width * sizeof(int16_t));
   orc_splat_s16_ns (dest + srcframe->components[component].width,
       dest[srcframe->components[component].width - 1],
       frame->components[component].width -
       srcframe->components[component].width);
-#else
-  oil_memcpy (dest, src,
-      srcframe->components[component].width * sizeof(int16_t));
-  oil_splat_s16_ns (dest + srcframe->components[component].width,
-      dest + srcframe->components[component].width - 1,
-      frame->components[component].width -
-      srcframe->components[component].width);
-#endif
 }
 
 SchroFrame *
